;Test fr DA-Wandler MAX517 und AD-Wandler MAX144
;D/A Ausgabe einer Rampe
;A/D Kanal 0 einlesen der Rampe 
;A/D Kanal 1 einlesen der Temperatur 
;Ausgabe ber serielle Schnittstelle
;Antwort auf seriellen Empfang 9600Baud
;Quarzfrequenz:  24 MHz

$include (at898252.inc)	        ;Prozessordefinition: AT89S8252

dacd    data    20h             ;Datum fr D/A-Wandler
sda	bit     p1.2            ;MAX518 serial data	
scl  	bit     p1.3            ;MAX518 serial clock
adcs    bit     p3.5            ;MAX144 chip select	

        org     0000H           ;Speicheradresse fr Programm
        mov     sp,     #7fh    ;Stack im indirekt-adr.int.RAM ab 80h
        acall   serini          ;Initialisierung serielle Schnittstelle

Haupt:  mov     r7,#255         ;Zhler fr Verzgerung
w1:     mov     r6,#255
w2:     jnb     ri,w3           ;Zeichen empfangen?
        acall   text
w3:     djnz    r6,w2        
        djnz    r7,w1
        cpl     p1.0
        jb      p1.0,   w4      ;abhngig vom Blinker alle anderen Ports
        mov     p0,     #0AAh   ; ansteuern
        mov     p2,     #055h
        clr     p1.1
        setb    p3.2
        clr     p3.3
        setb    p3.4
        clr     p3.6
        setb    p3.7
        sjmp    w5
w4:     mov     p2,     #0AAh   ; ansteuern
        mov     p0,     #055h
        setb    p1.1
        setb    p3.3
        clr     p3.2
        setb    p3.6
        clr     p3.4
        clr     p3.7     
w5:     acall	dac		;DA-Wandler
        acall   adc             ;AD-Wandler
	sjmp    Haupt

;------ DA-Wandler MAX157: Rampe aufsteigend
dac:    inc     dacd
        mov     a,      dacd    ;Ausgabebyte im Akku
        acall   dac_t           ;Ausgabe zum DAC
        ret
        
dac_t:  push    acc             ;Ausgabebyte retten
        setb    scl             ;bertragung Start
        setb    sda             ;Ausgangszustand - Timing fr 24MHz
        nop
        clr     sda             ;Startbedingung sda:1->0 bei scl-1
        nop                     ;min. 0,6s
        clr     scl		
        mov     a, #01011110b   ;adress byte:  010-Bausteine MAX517/18/19 ...
        acall   dac_t1          ; ... 11-MAX517/18 11-fr_Adr3 0-immer    
        mov     a, #00000000b   ;command byte: 000-immer 1-reset 1-powerdown
                                ; ... 00-beliebig  Kanal-Nr.0
        acall   dac_t1          ;
        pop     acc             ;Ausgabebyte transferieren
        acall   dac_t1
        setb    scl             ;bertragung Stop
        nop                     ; ... min. 0,6s spter ...
        setb    sda             ;bus free time tBUS:min.1,3s
        ret

dac_t1: mov     r4,     #9      ;bertragung(24MHz) Byte in Akku
dac_t2: clr     c
        rlc     a               ;Ausgabebit isolieren und ...
        mov     sda,    c       ; ... auf die Datenleitung
        setb    scl             ;min.100ns spter clock-1
        nop                     ;clock tHIGH:min.0,6s 
        clr     scl		;clock tLOW :min.1,3s 
        djnz    r4,     dac_t2
        ret

;------ AD-Wandler MAX144: 
adc:    acall   adc_t           ;AD-Wandler Ergebnis in b und a
        jb      b.4,    adc     ;wenn Kanal 1: nochmal wandeln (Kanal0)
        push    acc             ;LSBs retten
        mov     a,      b
        acall   aus             ;Byte als zwei Hexzahlen ausgeben
        pop     acc
        acall   aus
        mov     dptr,   #hex-2  ;Leertaste
        acall   seraus
        acall   adc_t           ;Kanal1 wandeln
        push    acc             ;LSBs retten
        mov     a,      b
        acall   aus             ;Byte als zwei Hexzahlen ausgeben
        pop     acc
        acall   aus
        mov     dptr,   #hex+1  ;neue Zeile
        acall   seraus
        ret
        
adc_t:  setb    adcs            ;Ausgangszustand - Timing fr 24MHz
        setb    sda             
        setb    scl             ;scl-1 -> Internal Clock Mode
        clr     adcs            ;ADC aktivieren
        clr     a               ;SRG fr Datenempfang lschen
        nop
        nop                     ;tWAKE min.2.5s
        nop
        nop
        clr     scl		;Wandlung starten
        nop                     ;... nach 120ns MAX144: sda=0
        jnb     sda,    $       ;sda=1 - EOC (end of conversion) max.7s
        mov     r4,     #3      ;3 Bit vergessen (sind 1)
adc_t1: setb    scl
        clr     scl
        djnz    r4,     adc_t1
        mov     r4,     #5      ;5 Bit einlesen (Kanal-Nr., D11 ... D8)
        acall   adc_t2
        mov     b,      a       ;MSBs retten
        mov     r4,     #8      ;8 Bit einlesen (D7 ... D0)
        acall   adc_t2
        setb    scl             ;Ruhezustand
        setb    adcs
        ret
adc_t2: setb    scl
        mov     c,      sda     ;Datenbit holen ...
        clr     scl
        rlc     a               ;... ins SRG
        djnz    r4,     adc_t2
        ret

;------ Tastenecho
text:   mov     a,      sbuf    ;Zeichen holen
        clr     ri
        push    acc             ;Zeichen retten
        acall   wti             ;und Echo
        mov     dptr,#text1     ;DatenZeiger auf Text setzen
        acall   seraus          ;Serielle Ausgabe aufrufen
        pop     acc
        acall   aus
        mov     dptr,   #hex    ;neue Zeile
        acall   seraus
        ret

;------ UP fr RS232-Initialisierung: 9600Bd, 8data, no parity, 1stop
serini: anl     tmod,   #0fh    ;Init serielle Schnittstelle mit Timer1
        orl     tmod,   #20h    ;... im Reload-Modus
        mov     th1,    #256-13 ;... Reload-Wert fr Timer1     (256-13) 
        orl     pcon,   #80h    ;... smod=1  9600Bd <- osz/(12*1*16 *13)
        mov     scon,   #5Ah    ;... Init RS232
        setb    ti              ;... scon.1, sendebereit
        setb    tr1             ;... Timer 1 ein
        ret

;------ UP fr RS232-Textausgabe: Textanfang im DPTR & letztes Textzeichen ...
seraus:                         ;... muss 00h (ASCII: NUL) sein!
        clr     a               ;Akku fr Ladebefehl lschen
        movc    a,@a+dptr       ;nchstes Zeichen aus dem Codespeicher holen
        jz      serend
w_ti:   jnb     ti,w_ti         ;warten bis Sende-Schieberegister leer
        mov     sbuf,a          ;... nchstes Zeichen -> Schieberegister
        clr     ti              ;TransferEnde-Bit lschen und ...
        inc     dptr            ;Zeiger fr Ausgabezeichen erhhen
        sjmp    seraus          ;wdhl. solange Zeichen nicht 00h war
serend: ret
        
text1:  db      '   Hallo, das war ein ',0 ;letztes Z.:00h
hex:    db      'h',0dh,0ah,0

aus:    push    acc             ;Hexzahl sichern
        anl     a,#0f0h         ;hherwertiges Halbbyte isolieren
        swap    a               ;Akku: X0 -> 0X (Halbbyte steht nun rechts)
        acall   hexv24          ;Halbbyte als ASCII-Zeichen ausgeben
        pop     acc
        anl     a,#00fh         ;niederwertiges Halbbyte isolieren
        acall   hexv24          ;Halbbyte als ASCII-Zeichen ausgeben
        ret

hexv24: mov     dptr,#tab       ;Zeiger auf Tabelle (Akku enthlt HEX-Wert!)
        movc    a,@a+dptr       ;x.Zeichen aus Tabelle holen (x-1 im Akku!)
wti:    jnb     ti,wti          ;letzte serielle Ausgabe abgeschlossen?
        mov     sbuf,a          ;Zeichen ausgeben
        clr     ti              ;Flag lschen
        ret
tab:    db      '0123456789ABCDEF'      ;Tabelle fr ASCII-HEX-Ziffern

        end
